/*
 * ruby pseudo terminal module
 * $Id: tpty.c,v 1.3 2002/11/03 23:52:49 tommy Exp $
 */
#define _XOPEN_SOURCE	1

#include <stdlib.h>
#include <fcntl.h>

#include "ruby.h"
#include "rubyio.h"

static VALUE cPty;

static VALUE make_file_obj(int fd)
{
    OpenFile *fp;
    NEWOBJ(file, struct RFile);
    OBJSETUP(file, rb_cIO, T_FILE);

    MakeOpenFile(file, fp);
    fp->f = rb_fdopen(fd, "r");
    fp->f2 = rb_fdopen(dup(fd), "w");
    fp->mode = FMODE_READWRITE;
    rb_io_synchronized(fp);
    return (VALUE)file;
}

#if defined(HAVE_GRANTPT)
static VALUE initialize(VALUE obj)
{
    int m, s;
    VALUE mfile, sfile;
    char *pn;

    if ((m = open("/dev/ptmx", O_RDWR)) < 0)
	rb_sys_fail("/dev/ptmx");
    if (grantpt(m) < 0) {
	close(m);
	rb_sys_fail("grantpt");
    }
    if (unlockpt(m) < 0) {
	close(m);
	rb_sys_fail("unlockpt");
    }
    if ((pn = ptsname(m)) == NULL) {
	close(m);
	rb_sys_fail("ptsname");
    }
    if ((s = open(pn, O_RDWR)) < 0) {
	close(m);
	rb_sys_fail(pn);
    }
    mfile = make_file_obj(m);
    sfile = make_file_obj(s);
    if (rb_block_given_p()) {
	rb_ensure(rb_yield, rb_ary_new3(2, sfile, mfile), rb_io_close, sfile);
	rb_io_close(sfile);
	sfile = Qnil;
    }
    rb_iv_set(obj, "slave", sfile);
    rb_iv_set(obj, "master", mfile);
    return Qnil;
}
#elif defined(HAVE_OPENPTY)
static VALUE initialize(VALUE obj)
{
    int m, s;
    VALUE mfile, sfile;
    char *pn;

    if (openpty(&m, &s, NULL, NULL, NULL) < 0) {
	rb_sys_fail("openpty");
    }
    mfile = make_file_obj(m);
    sfile = make_file_obj(s);
    rb_iv_set(obj, "master", mfile);
    rb_iv_set(obj, "slave", sfile);
    return Qnil;
}
#endif

static VALUE get_master(VALUE obj)
{
    return rb_iv_get(obj, "master");
}

static VALUE get_slave(VALUE obj)
{
    return rb_iv_get(obj, "slave");
}

void Init_tpty(void)
{
    cPty = rb_define_class("TPty", rb_cObject);
    rb_define_method(cPty, "initialize", initialize, 0);
    rb_define_method(cPty, "master", get_master, 0);
    rb_define_method(cPty, "slave", get_slave, 0);
}
